--- a tic tac toe board.
CREATE TABLE tttBoard
(
	spaceId tinyint not null primary key 
		check ([spaceId] between 1 and 9),
	[value] CHAR(1) not null default (' ') 
		check ([value] in ('X', 'O', ' ')),
)	
GO

--- a look up table to find the best move.
CREATE TABLE tttMoves
(
	id smallint primary key not null,
	bestNextMove tinyint not null,
	turn CHAR(1) default('X') not null
)
GO

-- populate the best move table.
BULK INSERT tttMoves
FROM 'C:\TEMP\tttMoves.bcp' 
WITH (DATAFILETYPE = 'native')
GO

-- creates a clean board
CREATE PROCEDURE s_CleanBoard AS
BEGIN
	DELETE tttBoard 
	INSERT tttBoard(spaceId) 
	VALUES (1), (2), (3), (4), (5), (6), (7), (8), (9);
END
GO

--- prints out the 
CREATE PROCEDURE s_PrintBoard
AS
BEGIN
	DECLARE @xo1 char(1), @xo2 char(1), @xo3 char(1), 
		@xo4 char(1), @xo5 char(1), @xo6 char(1),
		@xo7 char(1), @xo8 char(1), @xo9 char(1)
	SELECT @xo1 = value	FROM tttBoard WHERE spaceId = 1;
	SELECT @xo2 = value	FROM tttBoard WHERE spaceId = 2;
	SELECT @xo3 = value	FROM tttBoard WHERE spaceId = 3;
	SELECT @xo4 = value	FROM tttBoard WHERE spaceId = 4;
	SELECT @xo5 = value	FROM tttBoard WHERE spaceId = 5;
	SELECT @xo6 = value	FROM tttBoard WHERE spaceId = 6;
	SELECT @xo7 = value	FROM tttBoard WHERE spaceId = 7;
	SELECT @xo8 = value	FROM tttBoard WHERE spaceId = 8;
	SELECT @xo9 = value	FROM tttBoard WHERE spaceId = 9;
	PRINT @xo1 + '|' + @xo2 + '|' + @xo3;
	PRINT '-----'
	PRINT @xo4 + '|' + @xo5 + '|' + @xo6;
	PRINT '-----'
	PRINT @xo7 + '|' + @xo8 + '|' + @xo9;
END
GO

CREATE PROCEDURE s_MakeMove(@spaceId tinyint, @letter CHAR(1))
AS 
BEGIN 
	UPDATE tttBoard
	SET value = @letter
	WHERE spaceId = @spaceId
END
GO

CREATE PROCEDURE s_ComputerMove
AS
BEGIN
	DECLARE @BoardHash int 
	SELECT @BoardHash = SUM( POWER(3, spaceid - 1) *
		CASE value WHEN ' ' THEN 2 WHEN 'O' THEN 1 WHEN 'X' THEN 0 END)
	FROM tttBoard;
	
	DECLARE @Turn CHAR(1), @BestMove int
	SELECT @Turn = turn, @BestMove = bestNextMove
	FROM tttMoves
	WHERE id = @BoardHash;
	
	UPDATE tttBoard
	SET value = @Turn
	WHERE spaceId = @BestMove
	
END
GO

